@@ -1,12 +1,11 @@
 package Memcached::Client::Compressor;
 BEGIN {
-  $Memcached::Client::Compressor::VERSION = '1.99_03';
+  $Memcached::Client::Compressor::VERSION = '1.06';
 }
 #ABSTRACT: Abstract Base Class For Memcached::Client Compressor
 
 use strict;
 use warnings;
-use Memcached::Client::Log qw{LOG};
 
 
 sub new {
@@ -33,14 +32,6 @@ sub compress {
     die "You must implement compress";
 }
 
-
-sub log {
-    my ($self, $format, @args) = @_;
-    my $prefix = ref $self || $self;
-    $prefix =~ s,Memcached::Client::Compressor::,Compressor/,;
-    LOG ("$prefix> " . $format, @args);
-}
-
 1;
 
 __END__
@@ -52,7 +43,7 @@ Memcached::Client::Compressor - Abstract Base Class For Memcached::Client Compre
 
 =head1 VERSION
 
-version 1.99_03
+version 1.06
 
 =head1 SYNOPSIS
 
@@ -81,11 +72,6 @@ C<compress()> will (if the compression code is loadable) compress the
 data it is given, and if the data is large enough and the savings
 significant enough, it will compress it as well.
 
-=head2 C<log>
-
-Log the specified message with an appropriate prefix derived from the
-class name.
-
 =head1 AUTHOR
 
 Michael Alan Dorman <mdorman@ironicdesign.com>
