@@ -1,6 +1,6 @@
 package Memcached::Client::Compressor::Gzip;
 BEGIN {
-  $Memcached::Client::Compressor::Gzip::VERSION = '1.99_03';
+  $Memcached::Client::Compressor::Gzip::VERSION = '1.06';
 }
 #ABSTRACT: Implements Memcached Compression using Gzip
 
@@ -17,49 +17,50 @@ use constant +{
 };
 
 sub decompress {
-    my ($self, $data, $flags) = @_;
+    my ($self, $tuple) = @_;
 
-    return unless defined $data;
+    return unless defined $tuple->{data};
 
-    $flags ||= 0;
+    $tuple->{flags} ||= 0;
 
-    if ($flags & F_COMPRESS && HAVE_ZLIB) {
-        $self->log ("Uncompressing data") if DEBUG;
-        $data = Compress::Zlib::memGunzip ($data);
+    if ($tuple->{flags} & F_COMPRESS && HAVE_ZLIB) {
+        DEBUG "Uncompressing data";
+        $tuple->{data} = Compress::Zlib::memGunzip ($tuple->{data});
     }
 
-    return ($data, $flags);
+    return $tuple;
 }
 
 sub compress {
-    my ($self, $data, $flags) = @_;
+    my ($self, $tuple, $command) = @_;
 
-    $self->log ("Entering compress") if DEBUG;
-    return unless defined $data;
+    DEBUG "Entering compress";
+    return unless defined $tuple->{data};
 
-    $self->log ("Have data") if DEBUG;
-    my $len = bytes::length ($data);
+    DEBUG "Have data";
+    my $len = bytes::length ($tuple->{data});
 
-    $self->log ("Checking for Zlib") if DEBUG;
+    DEBUG "Checking for Zlib";
 
     if (HAVE_ZLIB) {
 
-        my $compressable = $self->{compress_threshold} && $len >= $self->{compress_threshold};
+        DEBUG "Checking for compressable (threshold $self->{compress_threshold}, command $command)";
+        my $compressable = ($command ne 'append' && $command ne 'prepend') && $self->{compress_threshold} && $len >= $self->{compress_threshold};
 
         if ($compressable) {
-            $self->log ("Compressing data") if DEBUG;
-            my $c_val = Compress::Zlib::memGzip ($data);
+            DEBUG "Compressing data";
+            my $c_val = Compress::Zlib::memGzip ($tuple->{data});
             my $c_len = bytes::length ($c_val);
 
             if ($c_len < $len * (1 - COMPRESS_SAVINGS)) {
-                $self->log ("Compressing is a win") if DEBUG;
-                $data = $c_val;
-                $flags |= F_COMPRESS;
+                DEBUG "Compressing is a win";
+                $tuple->{data} = $c_val;
+                $tuple->{flags} |= F_COMPRESS;
             }
         }
     }
 
-    return ($data, $flags);
+    return $tuple;
 }
 
 1;
@@ -73,7 +74,7 @@ Memcached::Client::Compressor::Gzip - Implements Memcached Compression using Gzi
 
 =head1 VERSION
 
-version 1.99_03
+version 1.06
 
 =head1 AUTHOR
 
