@@ -1,6 +1,6 @@
 package Memcached::Client::Connection;
 BEGIN {
-  $Memcached::Client::Connection::VERSION = '1.05';
+  $Memcached::Client::Connection::VERSION = '1.06';
 }
 # ABSTRACT: Class to manage Memcached::Client server connections
 
@@ -50,7 +50,7 @@ sub connect {
                                                    } elsif ($message eq "Connection timed out" and ++$self->{attempts} < 5) {
                                                        DEBUG "Requeueing connection timeout for %s", $self->{server};
                                                        delete $self->{handle};
-                                                       $self->connect ();
+                                                       $self->connect ($callback);
                                                    } else {
                                                        INFO "C [%s]: %s, %d attempts, %d completed, %d pending, %f last", $self->{server}, $message, $self->{attempts}, $self->{requests}, $pending, $last;
                                                        $callback->() if ($callback);
@@ -62,7 +62,7 @@ sub connect {
                                                    my ($handle) = @_;
                                                    DEBUG "C [%s]: preparing handle", $self->{server};
                                                    $self->{prepare}->($handle);
-                                                   return $self->{connect_timeout} || 5;
+                                                   return $self->{connect_timeout} || 0.5;
                                                },
                                                peername => $self->{server});
 }
@@ -126,7 +126,7 @@ Memcached::Client::Connection - Class to manage Memcached::Client server connect
 
 =head1 VERSION
 
-version 1.05
+version 1.06
 
 =head1 SYNOPSIS
 
