@@ -1,30 +1,30 @@
 package Memcached::Client::Serializer::JSON;
 BEGIN {
-  $Memcached::Client::Serializer::JSON::VERSION = '1.99_03';
+  $Memcached::Client::Serializer::JSON::VERSION = '1.06';
 }
 #ABSTRACT: Implements Memcached Serializing using JSON
 
 use strict;
 use warnings;
-use Memcached::Client::Log qw{DEBUG LOG};
+use Memcached::Client::Log qw{DEBUG};
 use JSON qw{decode_json encode_json};
 use base qw{Memcached::Client::Serializer};
 
 use constant F_JSON => 4;
 
 sub deserialize {
-    my ($self, $data, $flags) = @_;
+    my ($self, $tuple) = @_;
 
-    return unless defined $data;
+    return unless defined $tuple->{data};
 
-    $flags ||= 0;
+    $tuple->{flags} ||= 0;
 
-    if ($flags & F_JSON) {
-        $self->log ("Deserializing data") if DEBUG;
-        $data = decode_json $data;
+    if ($tuple->{flags} & F_JSON) {
+        DEBUG "Deserializing data";
+        $tuple->{data} = decode_json $tuple->{data};
     }
 
-    return $data;
+    return $tuple;
 }
 
 sub serialize {
@@ -32,21 +32,17 @@ sub serialize {
 
     return unless defined $data;
 
-    my $flags = 0;
+    my $tuple = {flags => 0};
 
     if (ref $data) {
-        $self->log ("Serializing data") if DEBUG;
-        $data = encode_json $data;
-        $flags |= F_JSON;
+        DEBUG "Serializing data";
+        $tuple->{data} = encode_json $data;
+        $tuple->{flags} |= F_JSON;
+    } else {
+        $tuple->{data} = $data;
     }
 
-    return ($data, $flags);
-}
-
-
-sub log {
-    my ($self, $format, @args) = @_;
-    LOG ($format, @args);
+    return $tuple;
 }
 
 1;
@@ -60,11 +56,7 @@ Memcached::Client::Serializer::JSON - Implements Memcached Serializing using JSO
 
 =head1 VERSION
 
-version 1.99_03
-
-=head1 METHODS
-
-=head2 log
+version 1.06
 
 =head1 AUTHOR
 
